<?php
namespace Ma\Service;

use Ma\Sys\LoggerFormatter;
use Monolog\Logger as NativeLogger;
use Monolog\Handler\StreamHandler;
use Monolog\Level;
use Exception;

/**
 * @package         Ma\Service
 * @class           Logger
 * @author          Mahdi Abu Yasmine
 * @date            2019-10-18 12:18:04 CET
 * @copyright       meta-tech.academy
 */
class Logger
{
    protected array $loggers;
    protected array $streams;
    protected LoggerFormatter $formatter;

    /**
     * @public
     * @constructor
     * @param       array   $config
     * @throws     \Exception
     */
    public function __construct(array $config)
    {
        if (empty($config)) {
            throw new Exception("missing config, require main  key : 'log' for ".__class__);
        }
        if (!isset($config['main'])) {
            throw new Exception("missing config key 'main' for ".__class__);
        }
        $name   = $config['main']['name'] ?? null;
        $level  = @Level::fromName($config['main']['level']);
        $path   = @$config['path'];
        $group  = @$config['group'];
        $gplvl  = @Level::fromName($config['group']['level']);
        $gplist = @$config['group']['list'];
;
        if (empty($name)) {
            $name  = 'main';
        }
        if (empty($level)) {
            $level = NativeLogger::WARNING;
        }
        if (!empty(@$config['formatter'])) {
            $this->setFormatter($config['formatter']);
        }
        $this->addLogger($name, $path, $level->value);
        if (!empty($bindLevel = intval(@$config['main']['bind']))) {
            $this->addLogger('main', $path, $bindLevel->value, true);
        }
        if (!empty($gplist)) {
            if (empty($gplvl)) {
                $gplvl = NativeLogger::WARNING;
            }
            foreach($gplist as $name) {
                if ($name != 'main') {
                    $this->addLogger($name, $path, $gplvl->value);
                }
            }
        }
    }

    /**
     * @public
     * @param       string      $name
     * @param       string      $path
     * @param       int         $level
     * @param       bool        $bind
     */
    public function addLogger(string $name, string $path, int $level, bool $bind = false)
    {
        $this->streams[$name.($bind ? '.bind' : '')] = new StreamHandler("$path$name.log", $level);
        if (!empty($this->formatter)) {
            $this->streams[$name]->setFormatter($this->formatter);
        }
        if (!$bind) {
            $this->loggers[$name] = new NativeLogger($name);
            $this->loggers[$name]->pushHandler($this->streams[$name]);
            if (isset($this->streams['main.bind'])) {
                $this->loggers[$name]->pushHandler($this->streams['main.bind']);
            }
        }
    }

    /**
     * @public
     * @param       array   $config
     */
    public function setFormatter(array $config = [])
    {
        if (empty(@$config['date'])) {
            $config['date'] = null;
        }
        if (empty(@$config['format'])) {
            $config['format'] = null;
        }
        $this->formatter = new LoggerFormatter($config['format'], $config['date']);
    }

    /**
     * @public
     * @param       callable    $callable
     */
    public function setExtra(callable $callable)
    {
        foreach($this->loggers as $logger) {
            $logger->pushProcessor(function ($record) use($callable) {
                $record['extra'] += $callable();
                return $record;
            });
        }
    }

    /**
     * @public
     * @param       string      $name
     * @return    ?\Monolog\Logger
     */
    public function get(string $name): ?NativeLogger
    {
        return $this->loggers[$name] ?? null;
    }
}
