<?php
namespace Ma\Service;

use Ma\App;
use Twig\Loader\FilesystemLoader;
use Twig\Environment;
use Twig\Extension\DebugExtension;
use Exception;

/**
 * @package         Ma\Service
 * @class           Tpl
 * @author          Mahdi Abu Yasmine
 * @date            2024-02-16 23:03:44 CET
 * @copyright       meta-tech.academy
 */
class Tpl
{
    protected array $config;
    protected array $tpls;

    /**
     * @public
     * @constructor
     * @param       array   $config
     */
    public function __construct(array $config)
    {
        $this->config = $config;
    }

    /**
     * @public
     * @param       Ma\App  $app
     * @param       array   $config
     * @return      Twig\Environment
     */
    public function newTpl(App $app, array $config): Environment
    {
        $loader = new FilesystemLoader($config['path']);
        $twig = new Environment($loader, [
            'cache' => $config['cache'],
            'debug' => $config['debug'],
        ]);
        $twig->addExtension(new DebugExtension());
        $twig->addGlobal('app', $app);
        return $twig;
    }

    /**
     * @public
     * @param      ?string  $name
     * @return      Twig\Environment
     */
    public function get(?string $name): Environment
    {
        global $app;
        if (empty($name)) {
            $name = 'default';
        }
        if (!isset($this->tpls[$name])) {
            $this->tpls[$name] = $this->newTpl($app, $this->config);
        }
        return $this->tpls[$name];
    }
}
