<?php
namespace Ma\Sys;

use Ma\Sys\Dbg;
use Ma\Util\Swiss;
use Monolog\Formatter\LineFormatter;
use Monolog\Formatter\NormalizerFormatter;
use Monolog\LogRecord;

/**
 * @package         Ma\Sys
 * @static
 * @class           LoggerFormatter
 * @extends         \Monolog\Formatter\LineFormatter
 * @author          Mahdi Abu Yasmine
 * @date            2019-10-18 17:14:12 CET
 * @copyright       meta-tech.academy
 */
class LoggerFormatter extends LineFormatter
{
    /**
     * @method      format
     * @public
     * @param       array $record
     * @return      string
     */
    public function format(LogRecord $record): string
    {
        $vars   = NormalizerFormatter::format($record);
        $output = $this->format;
        foreach ($vars['extra'] as $var => $val) {
            if (false !== strpos($output, '%extra.'.$var.'%')) {
                $output = str_replace('%extra.'.$var.'%', $this->stringify($val), $output);
                unset($vars['extra'][$var]);
            }
        }
        foreach ($vars['context'] as $var => $val) {
            if (false !== strpos($output, '%context.'.$var.'%')) {
                $output = str_replace('%context.'.$var.'%', $this->stringify($val), $output);
                unset($vars['context'][$var]);
            }
        }
        if ($this->ignoreEmptyContextAndExtra) {
            if (empty($vars['context'])) {
                unset($vars['context']);
                $output = str_replace('%context%', '', $output);
            }
            if (empty($vars['extra'])) {
                unset($vars['extra']);
                $output = str_replace('%extra%', '', $output);
            }
        }
        $of = Dbg::getFormatter();
        foreach ($vars as $var => $val) {
            if (false !== strpos($output, '%'.$var.'%')) {
                if ($var == 'datetime') {
                    $str = $of->apply ('arg', '[')
                         . $of->apply('date' , Swiss::formatDate($val, $this->dateFormat, 'Y-m-d '))
                         . $of->apply('time' , Swiss::formatDate($val, $this->dateFormat, 'H:i:s'))
                         . $of->apply('micro', Swiss::formatDate($val, $this->dateFormat, '.u'))
                         . $of->apply ('arg', ']');
                    $output = str_replace('%'.$var.'%', $str, $output);
                }
                else {
                    switch($var) {
                        case 'context' :
                            $val = self::parseParams($val, 'log_param_name', 'log_param_bracket', 'log_param_value');
                            break;
                        case 'extra' :
                            $val = self::parseParams($val, 'log_extra_name', 'log_extra_bracket', 'log_extra_value');
                            break;
                    }
                    $output = str_replace('%'.$var.'%', $of->apply($var, $this->stringify($val)), $output);
                }
            }
        }
        // remove leftover %extra.xxx% and %context.xxx% if any
        if (false !== strpos($output, '%')) {
            $output = preg_replace('/%(?:extra|context)\..+?%/', '', $output);
        }
        return $output . PHP_EOL;
    }

    /**
     * @private
     * @static
     * @param      ?array   $params
     * @param       string  $styleKey
     * @param       string  $styleVal
     * @param       string  $styleSep
     * @return      string
     */
    private static function parseParams (array $params = null, string $styleKey = 'key', string $styleVal = 'val', string $styleSep = 'sep'): string
    {
        $of  = Dbg::getFormatter();
        $str = $sep = "";
        if (!is_null($params)) {
            foreach($params as $k => $v) {
                $val = $v;
                if (is_array($v) || is_object($v)) {
                    $val = json_encode($v);
                }
                elseif (is_bool($v)) {
                    $val = $v ? 'true' : 'false';
                }
                $str .= $of->apply($styleSep, $sep) . $of->apply($styleKey, $k) . $of->apply($styleSep, ':') 
                      . $of->apply($styleVal, is_null($val) ? 'null' : $val);
                if (empty($sep)) $sep = ', ';
            }
        }
        return $of->apply($styleSep, '[') . $str . $of->apply($styleSep, ']');
    }

}
