<?php
namespace Ma\Sys;

use Ma\Sys\TimeIt;
use Ma\Util\Observable;
use SplObserver;
use SplSubject;
use InvalidArgumentException;

/**
 * @package         Ma\Sys
 * @class           MemoryObserver
 * @implements      \SplObserver
 * @author          Mahdi Abu Yasmine
 * @date            2019-08-22 11:29:10 CET
 * @copyright       meta-tech.academy
 * @license         GNU GPL v3
 */
class MemoryObserver implements SplObserver
{
    protected array $data;

    /**
     * @constructor
     * @public
     */
    public function __construct()
    {
        $this->data = [];
    }

    /**
     * @method      delta
     * @public
     * @param      ?string      $name
     * @param       bool        $peak
     * @return     ?int
     */
    public function delta(?string $name, bool $peak=false): ?int
    {
        $delta = null;
        if (is_null($name)) $name = TimeIt::MAIN_TAG;
        if (!empty($this->data[$name])) {
            $size     = count($this->data[$name]);
            $startTag = $size == 2 ? $name : TimeIt::MAIN_TAG;
            $endIndex = $size == 2 ? TimeIt::KEY_END : TimeIt::KEY_START;
            $delta    = $this->data[$name][$endIndex][intval($peak)]-$this->data[$startTag][TimeIt::KEY_START][intval($peak)];
        }
        return $delta;
    }

    /**
     * @method      update
     * @public
     * @param       \SplSubject $subject
     * @return      void
     */
    public function update(SplSubject $subject): void
    {
        if (!($subject instanceof TimeIt)) {
            throw new InvalidArgumentException("SplSubject subject parameter should be an instance of Ma\Sys\TimeIt");
        }
        $this->data[$subject->getLastTag()][$subject->getLastState()] = [memory_get_usage(), memory_get_peak_usage()];
    }
}
