<?php
namespace Ma\Sys;

use InvalidArgumentException;
/**
 * @package         Ma\Sys
 * @class           TermColor
 * @author          Mahdi Abu Yasmine
 * @date            2019-07-24 22:59:10 CET
 * @copyright       meta-tech.academy
 * @license         GNU GPL v3
 */
class TermColor
{
    protected int $r;
    protected int $g;
    protected int $b;

    /**
     * @constructor
     * @public
     * @param       int     $r      red value
     * @param       int     $g      green value
     * @param       int     $b      blue value
     */
    public function __construct(int $r=255, int $g=255, int $b=255)
    {
        $this->setRed($r);
        $this->setGreen($g);
        $this->setBlue($b);
    }

    /**
     * @method      code
     * @public
     * @return      string
     */
    public function code(): string
    {
        return sprintf("#%02X%02X%02X", $this->r, $this->g, $this->b);
    }

    /**
     * @method      checkValue
     * @private
     * @param       int     $value
     * @throws      \InvalidArgumentException
     */
    private function checkValue(int $value): void
    {
        if ($value < 0 || $value >255) throw new InvalidArgumentException("value $value must be between 0 and 255");
    }

    /**
     * @method      setRed
     * @public
     * @param       int     $value
     * @throws      \InvalidArgumentException
     */
    public function setRed(int $value): void
    {
        $this->checkValue($value);
        $this->r = $value;
    }

    /**
     * @method      setGreen
     * @public
     * @param       int     $value
     * @throws      \InvalidArgumentException
     */
    public function setGreen(int $value): void
    {
        $this->checkValue($value);
        $this->g = $value;
    }

    /**
     * @method      setBlue
     * @public
     * @param       int     $value
     * @throws      \InvalidArgumentException
     */
    public function setBlue(int $value): void
    {
        $this->checkValue($value);
        $this->b = $value;
    }

    /**
     * @method      getRed
     * @public
     * @return      int
     */
    public function getRed(): int
    {
        return $this->r;
    }

    /**
     * @method      getGreens
     * @public
     * @return      int
     */
    public function getGreen(): int
    {
        return $this->g;
    }

    /**
     * @method      getBlue
     * @public
     * @return      int
     */
    public function getBlue(): int
    {
        return $this->b;
    }

    /**
     * @method      toString
     * @public
     * @return      string
     */
    public function toString(): string
    {
        return sprintf("%s(r:%03u,g:%03u,b:%03u)", get_class($this), $this->r, $this->g, $this->b);
    }
}
