<?php
namespace Ma\Sys;

/**
 * @package         Ma\Sys
 * @static
 * @class           TermStyle
 * @author          Mahdi Abu Yasmine
 * @date            2019-07-24 22:59:10 CET
 * @copyright       meta-tech.academy
 * @license         GNU GPL v3
 */
class TermStyle
{
    protected ?TermColor $fg;
    protected ?TermColor $bg;
    protected array     $opt;

    const START_SEQ = "\033[";
    const OFF_SEQ   = "\033[m";

    /**
     *
     */
    public function __construct (?TermColor $foreground , ?TermColor $background, array $options = [TermOption::NORMAL])
    {
        $this->fg  = $foreground;
        $this->bg  = $background;
        $this->opt = $options;
    }

    /**
     * @method      apply
     * @public
     * @return      string
     */
    public function apply ():string
    {
        $str = self::START_SEQ;
        foreach ($this->opt as $o) {
            $str .= $o.';';
        }
        if ($this->fg != null) {
            $str .= sprintf ("38;2;%u;%u;%u%s", $this->fg->getRed(), $this->fg->getGreen(), $this->fg->getBlue(), $this->bg != null ? ";" : "");
        }
        if ($this->bg != null) {
            $str .= sprintf ("48;2;%u;%u;%u", $this->bg->getRed(), $this->bg->getGreen(), $this->bg->getBlue());
        }
        $str .= 'm';
        return $str;
    }

    /**
     * @method      toString
     * @public
     * @return      string
     */
    public function toString ():string
    {
        $str = '';
        if (count($this->opt) > 0) {
            foreach ($this->opt as $opt) {
                $str .= (strlen($str) > 0 ? ", " : "") . TermOption::getLabel($opt);
            }
        }
        return printf (
            "%s(fg:%s, bg:%s, opt:{%s})",
            get_class($this), 
            $this->fg != null ? $this->fg->toString() : "null",
            $this->bg != null ? $this->bg->toString() : "null",
            $str
        );
    }
    /**
     * @method      off
     * @public
     * @static
     * @return      string
     */
    public static function off ():string
    {
        return self::OFF_SEQ;
    }

    /**
     * @method      applied
     * @public
     * @static
     * @param      ?\Ma\Sys\TermColor   $fg         foreground color
     * @param      ?\Ma\Sys\TermColor   $bg         background color
     * @param       array               $options    optional options
     * @return      string
     */
    public static function applied (TermColor $fg = null, TermColor $bg = null, array $options = [TermOption::NORMAL]):string
    {
        return (new TermStyle($fg, $bg, $options))->apply();
    }
}
