<?php
namespace Ma\Util;

use Ma\Sys\Dbg;
// use Ma\Db\EntityDefinition;
use DateTime;
use InvalidArgumentException;

/**
 * @package         Ma\Db
 * @static
 * @class           Swiss
 * @author          Mahdi Abu Yasmine
 * @date            2019-07-24 22:59:10 CET
 * @copyright       iaas.studio
 * @license         GNU GPL v3
 */
class Swiss
{
    private const  UNIT_SIZE           = ['B','KiB','MiB','GiB','TiB','PiB'];

    /**
     * check if specified method name is a getter or setter method
     * @public
     * @static
     * @param       string      $name
     * @return      bool
     */
    public static function isGetterOrSetter(string $name): bool
    {
        return in_array(substr($name, 0, 3), ['get', 'set']);
    }

    /**
     * extract field name from getter or setter method
     *
     * @public
     * @static
     * @param       string      $name
     * @return     ?string
     */
    public static function getFieldNameFromGetSet(string $name): ?string
    {
        return self::isGetterOrSetter($name) ? strtolower($name[3]) . substr($name, 4) : null;
    }

    /**
     * get setter for specified $name
     *
     * @public
     * @static
     * @param       string      $name
     * @return      string
     */
    public static function getSetter(string $name): string
    {
        return 'set' . ucfirst($name) ;
    }

    /**
     * get getter for specified $name
     *
     * @public
     * @static
     * @param       string     $name
     * @return      string
     */
    public static function getGetter(string $name): string
    {
        return 'get' . ucfirst($name) ;
    }

    /**
     * @public
     * @static
     * @param       mixed   $int
     * @param       bool    $zero
     * @return      bool
     */
    public static function isValidId($int, bool $zero=false): bool
    {
        return $int !== null && (($int === 0 && $zero) || (intval($int) == "$int" && $int > 0));
    }

    /**
     * @public
     * @static
     * @param       array   $aint
     * @param       bool    $zero
     * @return      bool
     */
    public static function isValidIds(array $aint, bool $zero=false): bool
    {
        $done = true;
        foreach($aint as $int) {
            $done = $done && self::isValidId($int, $zero);
        }
        return $done;
    }

    /**
     * @public
     * @static
     * @param       array      &$array
     * @param       string      $prefix
     * @return      array
     */
    public static function array_extract_data(array &$array, string $prefix): array
    {
        $dtag = Dbg::in(__method__, Dbg::DEBUG_4, compact(Dbg::args(__method__)), __line__);
        $data = [];
        foreach($array as $k => $v) {
            //~ var_dump(compact('k', 'v'));
            if (substr($k, 0, strlen($prefix)) == $prefix) {
                $data[substr($k, strlen($prefix))] = $v;
                unset($array[$k]); 
            }
        }
        Dbg::out($dtag, __line__, $data);
        return $data;
    }

    /**
     * @public
     * @static
     * @param       string      $idPrefix
     * @return      string
     */
    public static function genid(string $idPrefix = 'hk76f4d'): string
    {
        return hash('sha256', time().uniqid($idPrefix, true));
    }

    /**
     * @public
     * @static
     * @param       int    $bytes
     * @param       int    $precision
     * @return      string
     */
    public static function readsize(int $bytes, int $precision=4): string
    {
        return ($bytes==0 ? 0 : @round($bytes/pow(1024,($i=floor(log($bytes,1024)))),$precision)) . 
               ' ' . 
               @self::UNIT_SIZE[$i] ?? 'B';
    }

    /**
     * @methdo      numbetween
     * @public
     * @static
     * @param       int     $num
     * @param       int     $low
     * @param       int     $high
     * @param       bool    $strict
     * @return      bool
     */
    public static function numbetween(int $num, int $low, int $high, bool $strict=false): bool
    {
        $c = $strict ? 0 : 1;
        return $num > $low - $c && $num < $high + $c;
    }

    /**
     * @method      formatDate
     * @public
     * @static
     * @param       string      $date
     * @param       string      $fromFormat
     * @param       string      $toFormat
     * @return     ?string
     */
    public static function formatDate(string $date, string $fromFormat='d-m-Y', string $toFormat='Y-m-d'): ?string
    {
        $dt = DateTime::createFromFormat($fromFormat, $date);
        return !$dt ? null : $dt->format($toFormat);
    }

    /**
     * @param       string      $input
     * @return      string
     * @throws InvalidArgumentException
     */
    public static function urlsafeB64Decode(string $input): string
    {
        if ($remainder = strlen($input) % 4) {
            $input .= str_repeat('=', 4 - $remainder);
        }
        return base64_decode(strtr($input, '-_', '+/'));
    }

    /**
     * @param       string      $input
     * @return      string
     */
    public static function urlsafeB64Encode(string $input): string
    {
        return str_replace('=', '', strtr(base64_encode($input), '+/', '-_'));
    }
}
