<?php
namespace Ma\Web;

use Ma\App;
use Ma\Web\ControllerInterface;

/**
 * @package         Ma\Web
 * @class           AbstractController
 * @author          Mahdi Abu Yasmine
 * @date            2024-02-16 23:34:56 CET
 * @copyright       meta-tech.academy
 */
abstract class AbstractController implements ControllerInterface
{
    /**
     * @public
     * @constructor
     * @param       Ma\App  $app
     * @param       string  $path
     */
    public function __construct(App $app, string $path)
    {
        $this->init($app);
        foreach ($this->connect() as $routeName => $methodName) {
            $app['router']->bind($path . $routeName, $this, $methodName);
        }
    }

    /**
     * @public
     * @param       Ma\App  $app
     */
    public function init(App $app)
    {
    }

    /**
     * @public
     * @abstract
     * @return      array
     */
    public abstract function connect(): array;

}
