<?php
namespace Ma\Web;

use Exception;

/**
 * @package         Ma\Web
 * @class           Route
 * @author          Mahdi Abu Yasmine
 * @date            2024-02-16 19:18:48 CET
 * @copyright       meta-tech.academy
 */
class Route
{
    public const VAR_START = "{";
    public const VAR_END   = "}";

    protected string $def;
    protected array $tokens;
    protected array $vars;

    /**
     * @public
     * @constructor
     * @param       string  $def
     * @throws     \Exception
     */
    public function __construct(string $def)
    {
        $this->def = $def;
        $this->read();
    }

    /**
     * @public
     * @return      string
     */
    public function getDef(): string
    {
        return $def;
    }

    /**
     * @public
     * @param       string  $name
     * @return      bool
     */
    public function isValidVarName(string $name): bool
    {
        return preg_match("/[a-zA-Z_\x7f-\xff][a-zA-Z0-9_\x7f-\xff]*/", $name) == 1;
    }

    /**
     * @public
     * @param       string  $path
     * @return      bool
     */
    public function isValidPath(string $path): bool
    {
        return str_starts_with($part, '/')
            && !str_contains($path, '?')
            && !str_contains($path, '#')
            && filter_var("p://h".$path, FILTER_VALIDATE_URL, FILTER_FLAG_PATH_REQUIRED) !== false;
    }

    /**
     * @protected
     * @throws     \Exception
     */
    protected function read()
    {
        $this->tokens = explode('/', $this->def);
        array_shift($this->tokens);
        $this->vars   = [];
        $vs = $ve = $name = false;
        foreach ($this->tokens as $i => $part) {
            $vs = str_starts_with($part, self::VAR_START);
            $ve = str_ends_with($part, self::VAR_END);
            if ($vs && $ve) {
                $name = substr($part, 1, -1);
                if (!$this->isValidVarName($name)) {
                    throw new Exception("invalid variable name at path part `$part` definition");
                }
                $this->vars[$i] = $name;
            }
        }
    }

    /**
     * @protected
     * @return      array
     */
    public function match($path): array
    {
        $vars  = [];
        $parts = explode('/', $path);
        array_shift($parts);
        $done  = count($parts) == count($this->tokens);
        if ($done) {
            foreach ($parts as $i => $part) {
                if (array_key_exists($i, $this->vars)) {
                    $vars[$this->vars[$i]] = $part;
                } else {
                    $done = $part == $this->tokens[$i];
                    if (!$done) {
                        break;
                    }
                }
            }
        }
        return compact('done', 'vars');
    }
}
