<?php
namespace Ma\Web;

use Ma\App;
use Ma\Sys\Dbg;
use Ma\Web\ControllerInterface;
use Exception;
use stdclass;

/**
 * @package         Ma\Web
 * @class           Router
 * @author          Mahdi Abu Yasmine
 * @date            2024-02-16 20:05:38 CET
 * @copyright       meta-tech.academy
 */
class Router
{
    protected array $routes;

    /**
     * @public
     * @constructor
     */
    public function __construct()
    {
        $this->routes = [];
    }

    /**
     * @protected
     * @param       Ma\Web\Route                $route
     * @param       Ma\Web\ControllerInterface  $ctrl
     * @param       string                      $method
     */
    protected function newBindRoute(Route $route, ControllerInterface $ctrl, string $method)
    {
        $o = new stdclass();
        $o->route  = $route;
        $o->ctrl   = $ctrl;
        $o->method = $method;
        return $o;
    }

    /**
     * @public
     * @param       string                      $path
     * @param       Ma\Web\ControllerInterface  $ctrl
     * @param       string                      $method
     */
    public function bind(string $path, ControllerInterface $ctrl, string $method)
    {
        $this->routes[] = $this->newBindRoute(new Route($path), $ctrl, $method);
    }

    /**
     * @public
     * @param       string  $path
     */
    public function resolve($path)
    {
        $content = null;
        foreach ($this->routes as $index => $o) {
            $rs = $o->route->match($path);
            if ($rs['done']) {
                $content = call_user_func_array(array($o->ctrl, $o->method), $rs['vars']);
                break;
            }
        }
        if (!empty($content)) {
            if (is_string($content)) {
                echo $content;
            }
        } else if (is_null($content)) {
            http_response_code(404);
            echo "page `$path` not found !";
        }
    }
}
