<?php
namespace Zic;

use Exception;

/**
 * @package         Ma\Web
 * @class           SessionManager
 * @author          Mahdi Abu Yasmine
 * @date            2024-02-17 04:09:12 CET
 * @copyright       meta-tech.academy
 */
class SessionManager
{
    protected array $config;

    private const SALT = "xL5!b8<àe3':oP";

    /**
     * @public
     * @constructor
     * @param       array   $string
     * @return      string
     */
    public function __construct(array $config = [])
    {
        $this->config = $config;
    }

    /**
     * @protected
     * @return      string
     */
    protected function getSalt(): string
    {
        return $this->config['salt'] ?: self::SALT;
    }

    /**
     * @protected
     * @return      string
     */
    protected function getUserId(): string
    {
        return hash('sha256', $this->getSalt() . $this->getUserAgent() . getUserIp());
    }

    /**
     * @public
     * @return     ?string
     */
    public function getUserAgent(): ?string
    {
        return $_SERVER['HTTP_USER_AGENT']?: null;
    }

    /**
     * @public
     * @return      string
     */
    public function getUserIp(): string
    {
        if (!empty($_SERVER['HTTP_CLIENT_IP'])) {
            $ip = $_SERVER['HTTP_CLIENT_IP'];
        } elseif (!empty($_SERVER['HTTP_X_FORWARDED_FOR'])) {
            $ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
        } else {
            $ip = $_SERVER['REMOTE_ADDR'];
        }
        return $ip;
    }

    /**
     * @public
     */
    public function start()
    {
        ini_set('session.use_strict_mode', 1);
        session_start();
        if (!empty($_SESSION['deleted_time']) && $_SESSION['deleted_time'] < time() - 180) {
            session_destroy();
            session_start();
        }
    }

    /**
     * @public
     */
    public function regenId()
    {
        if (session_status() != PHP_SESSION_ACTIVE) {
            session_start();
        }
        $_SESSION['deleted_time'] = time();
        session_commit();
        ini_set('session.use_strict_mode', 0);
        session_id($this->getUserId());
        session_start();
    }

}
